@@ -34,26 +34,39 @@ CONNECT_TIMEOUT = 15
 READ_TIMEOUT = 30
 
 LONG_POLLING_TIMEOUT = 10 # Should be positive, short polling should be used for testing purposes only (https://core.telegram.org/bots/api#getupdates)
-
+REQUEST_TIMEOUT = 10
+MAX_RETRIES = 3
 logger = telebot.logger
 
-RETRY_ON_ERROR = False
-RETRY_TIMEOUT = 2
-MAX_RETRIES = 15
-
 async def _process_request(token, url, method='get', params=None, files=None, request_timeout=None):
-    params = compose_data(params, files)
+    params = prepare_data(params, files)
+    if request_timeout is None:
+        request_timeout = REQUEST_TIMEOUT
+    timeout = aiohttp.ClientTimeout(total=request_timeout)
+    got_result = False
+    current_try=0
     async with await session_manager._get_new_session() as session:
-        async with session.request(method=method, url=API_URL.format(token, url), data=params, timeout=request_timeout) as response:
-            logger.debug("Request: method={0} url={1} params={2} files={3} request_timeout={4}".format(method, url, params, files, request_timeout).replace(token, token.split(':')[0] + ":{TOKEN}"))
-            json_result = await _check_result(url, response)
-            if json_result:
-                return json_result['result']
+        while not got_result and current_try<MAX_RETRIES-1:
+            current_try +=1
+            try:
+                response = await session.request(method=method, url=API_URL.format(token, url), data=params, timeout=timeout)
+                logger.debug("Request: method={0} url={1} params={2} files={3} request_timeout={4} current_try={5}".format(method, url, params, files, request_timeout, current_try).replace(token, token.split(':')[0] + ":{TOKEN}"))
+                json_result = await _check_result(url, response)
+                if json_result:
+                    return json_result['result']
+            except (ApiTelegramException,ApiInvalidJSONException, ApiHTTPException) as e:
+                raise e
+            except:
+                pass
+        if not got_result:
+            raise RequestTimeout("Request timeout. Request: method={0} url={1} params={2} files={3} request_timeout={4}".format(method, url, params, files, request_timeout, current_try))
+        
 
 
-def guess_filename(obj):
+
+def prepare_file(obj):
     """
-    Get file name from object
+    returns os.path.basename for a given file
 
     :param obj:
     :return:
@@ -63,9 +76,9 @@ def guess_filename(obj):
         return os.path.basename(name)
 
 
-def compose_data(params=None, files=None):
+def prepare_data(params=None, files=None):
     """
-    Prepare request data
+    prepare data for request.
 
     :param params:
     :param files:
@@ -85,7 +98,7 @@ def compose_data(params=None, files=None):
                 else:
                     raise ValueError('Tuple must have exactly 2 elements: filename, fileobj')
             else:
-                filename, fileobj = guess_filename(f) or key, f
+                filename, fileobj = prepare_file(f) or key, f
 
             data.add_field(key, fileobj, filename=filename)
 
@@ -228,7 +241,7 @@ async def send_message(
         token, chat_id, text,
         disable_web_page_preview=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None,
-        entities=None, allow_sending_without_reply=None):
+        entities=None, allow_sending_without_reply=None, protect_content=None):
     """
     Use this method to send text messages. On success, the sent Message is returned.
     :param token:
@@ -242,6 +255,7 @@ async def send_message(
     :param timeout:
     :param entities:
     :param allow_sending_without_reply:
+    :param protect_content:
     :return:
     """
     method_name = 'sendMessage'
@@ -262,6 +276,8 @@ async def send_message(
         params['entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(entities))
     if allow_sending_without_reply is not None:
         params['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        params['protect_content'] = protect_content
     
     return await _process_request(token, method_name, params=params)
 
@@ -333,19 +349,21 @@ async def get_chat_member(token, chat_id, user_id):
 
 async def forward_message(
         token, chat_id, from_chat_id, message_id,
-        disable_notification=None, timeout=None):
+        disable_notification=None, timeout=None, protect_content=None):
     method_url = r'forwardMessage'
     payload = {'chat_id': chat_id, 'from_chat_id': from_chat_id, 'message_id': message_id}
     if disable_notification is not None:
         payload['disable_notification'] = disable_notification
     if timeout:
         payload['timeout'] = timeout
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
 async def copy_message(token, chat_id, from_chat_id, message_id, caption=None, parse_mode=None, caption_entities=None,
                  disable_notification=None, reply_to_message_id=None, allow_sending_without_reply=None,
-                 reply_markup=None, timeout=None):
+                 reply_markup=None, timeout=None, protect_content=None):
     method_url = r'copyMessage'
     payload = {'chat_id': chat_id, 'from_chat_id': from_chat_id, 'message_id': message_id}
     if caption is not None:
@@ -364,13 +382,15 @@ async def copy_message(token, chat_id, from_chat_id, message_id, caption=None, p
         payload['allow_sending_without_reply'] = allow_sending_without_reply
     if timeout:
         payload['timeout'] = timeout
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
 async def send_dice(
         token, chat_id,
         emoji=None, disable_notification=None, reply_to_message_id=None,
-        reply_markup=None, timeout=None, allow_sending_without_reply=None):
+        reply_markup=None, timeout=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendDice'
     payload = {'chat_id': chat_id}
     if emoji:
@@ -385,6 +405,8 @@ async def send_dice(
         payload['timeout'] = timeout
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
@@ -392,7 +414,7 @@ async def send_photo(
         token, chat_id, photo,
         caption=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None,
-        caption_entities=None, allow_sending_without_reply=None):
+        caption_entities=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendPhoto'
     payload = {'chat_id': chat_id}
     files = None
@@ -418,13 +440,15 @@ async def send_photo(
         payload['caption_entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(caption_entities))
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_media_group(
         token, chat_id, media,
         disable_notification=None, reply_to_message_id=None,
-        timeout=None, allow_sending_without_reply=None):
+        timeout=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendMediaGroup'
     media_json, files = await convert_input_media_array(media)
     payload = {'chat_id': chat_id, 'media': media_json}
@@ -436,6 +460,8 @@ async def send_media_group(
         payload['timeout'] = timeout
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(
         token, method_url, params=payload,
         method='post' if files else 'get',
@@ -447,7 +473,7 @@ async def send_location(
         live_period=None, reply_to_message_id=None, 
         reply_markup=None, disable_notification=None, 
         timeout=None, horizontal_accuracy=None, heading=None,
-        proximity_alert_radius=None, allow_sending_without_reply=None):
+        proximity_alert_radius=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendLocation'
     payload = {'chat_id': chat_id, 'latitude': latitude, 'longitude': longitude}
     if live_period:
@@ -468,6 +494,8 @@ async def send_location(
         payload['disable_notification'] = disable_notification
     if timeout:
         payload['timeout'] = timeout
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
@@ -519,7 +547,7 @@ async def send_venue(
         foursquare_id=None, foursquare_type=None, disable_notification=None,
         reply_to_message_id=None, reply_markup=None, timeout=None,
         allow_sending_without_reply=None, google_place_id=None,
-        google_place_type=None):
+        google_place_type=None, protect_content=None):
     method_url = r'sendVenue'
     payload = {'chat_id': chat_id, 'latitude': latitude, 'longitude': longitude, 'title': title, 'address': address}
     if foursquare_id:
@@ -540,13 +568,15 @@ async def send_venue(
         payload['google_place_id'] = google_place_id
     if google_place_type:
         payload['google_place_type'] = google_place_type
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
 async def send_contact(
         token, chat_id, phone_number, first_name, last_name=None, vcard=None,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, timeout=None,
-        allow_sending_without_reply=None):
+        allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendContact'
     payload = {'chat_id': chat_id, 'phone_number': phone_number, 'first_name': first_name}
     if last_name:
@@ -563,6 +593,8 @@ async def send_contact(
         payload['timeout'] = timeout
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
@@ -576,7 +608,8 @@ async def send_chat_action(token, chat_id, action, timeout=None):
 
 async def send_video(token, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None,
                parse_mode=None, supports_streaming=None, disable_notification=None, timeout=None, 
-               thumb=None, width=None, height=None, caption_entities=None, allow_sending_without_reply=None):
+               thumb=None, width=None, height=None, caption_entities=None, allow_sending_without_reply=None,
+               protect_content=None):
     method_url = r'sendVideo'
     payload = {'chat_id': chat_id}
     files = None
@@ -616,13 +649,15 @@ async def send_video(token, chat_id, data, duration=None, caption=None, reply_to
         payload['caption_entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(caption_entities))
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_animation(
         token, chat_id, data, duration=None, caption=None, reply_to_message_id=None, reply_markup=None,
         parse_mode=None, disable_notification=None, timeout=None, thumb=None, caption_entities=None,
-        allow_sending_without_reply=None):
+        allow_sending_without_reply=None, width=None, height=None, protect_content=None):
     method_url = r'sendAnimation'
     payload = {'chat_id': chat_id}
     files = None
@@ -656,12 +691,18 @@ async def send_animation(
         payload['caption_entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(caption_entities))
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if width:
+        payload['width'] = width
+    if height:
+        payload['height'] = height
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_voice(token, chat_id, voice, caption=None, duration=None, reply_to_message_id=None, reply_markup=None,
                parse_mode=None, disable_notification=None, timeout=None, caption_entities=None,
-               allow_sending_without_reply=None):
+               allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendVoice'
     payload = {'chat_id': chat_id}
     files = None
@@ -687,11 +728,13 @@ async def send_voice(token, chat_id, voice, caption=None, duration=None, reply_t
         payload['caption_entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(caption_entities))
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_video_note(token, chat_id, data, duration=None, length=None, reply_to_message_id=None, reply_markup=None,
-                    disable_notification=None, timeout=None, thumb=None, allow_sending_without_reply=None):
+                    disable_notification=None, timeout=None, thumb=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendVideoNote'
     payload = {'chat_id': chat_id}
     files = None
@@ -723,12 +766,14 @@ async def send_video_note(token, chat_id, data, duration=None, length=None, repl
             payload['thumb'] = thumb
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_audio(token, chat_id, audio, caption=None, duration=None, performer=None, title=None, reply_to_message_id=None,
                reply_markup=None, parse_mode=None, disable_notification=None, timeout=None, thumb=None,
-               caption_entities=None, allow_sending_without_reply=None):
+               caption_entities=None, allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendAudio'
     payload = {'chat_id': chat_id}
     files = None
@@ -766,12 +811,14 @@ async def send_audio(token, chat_id, audio, caption=None, duration=None, perform
         payload['caption_entities'] = json.dumps(types.MessageEntity.to_list_of_dicts(caption_entities))
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload, files=files, method='post')
 
 
 async def send_data(token, chat_id, data, data_type, reply_to_message_id=None, reply_markup=None, parse_mode=None,
               disable_notification=None, timeout=None, caption=None, thumb=None, caption_entities=None,
-              allow_sending_without_reply=None, disable_content_type_detection=None, visible_file_name=None):
+              allow_sending_without_reply=None, disable_content_type_detection=None, visible_file_name=None, protect_content=None):
     method_url = await get_method_by_type(data_type)
     payload = {'chat_id': chat_id}
     files = None
@@ -1176,7 +1223,7 @@ async def delete_message(token, chat_id, message_id, timeout=None):
 async def send_game(
         token, chat_id, game_short_name,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, timeout=None,
-        allow_sending_without_reply=None):
+        allow_sending_without_reply=None, protect_content=None):
     method_url = r'sendGame'
     payload = {'chat_id': chat_id, 'game_short_name': game_short_name}
     if disable_notification is not None:
@@ -1189,6 +1236,8 @@ async def send_game(
         payload['timeout'] = timeout
     if allow_sending_without_reply is not None:
         payload['allow_sending_without_reply'] = allow_sending_without_reply
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
@@ -1253,7 +1302,7 @@ async def send_invoice(
         need_name=None, need_phone_number=None, need_email=None, need_shipping_address=None,
         send_phone_number_to_provider = None, send_email_to_provider = None, is_flexible=None,
         disable_notification=None, reply_to_message_id=None, reply_markup=None, provider_data=None,
-        timeout=None, allow_sending_without_reply=None, max_tip_amount=None, suggested_tip_amounts=None):
+        timeout=None, allow_sending_without_reply=None, max_tip_amount=None, suggested_tip_amounts=None, protect_content=None):
     """
     Use this method to send invoices. On success, the sent Message is returned.
     :param token: Bot's token (you don't need to fill this)
@@ -1285,6 +1334,7 @@ async def send_invoice(
     :param max_tip_amount: The maximum accepted amount for tips in the smallest units of the currency
     :param suggested_tip_amounts: A JSON-serialized array of suggested amounts of tips in the smallest units of the currency.
         At most 4 suggested tip amounts can be specified. The suggested tip amounts must be positive, passed in a strictly increased order and must not exceed max_tip_amount.
+    :param protect_content:
     :return:
     """
     method_url = r'sendInvoice'
@@ -1331,6 +1381,8 @@ async def send_invoice(
         payload['max_tip_amount'] = max_tip_amount
     if suggested_tip_amounts is not None:
         payload['suggested_tip_amounts'] = json.dumps(suggested_tip_amounts)
+    if protect_content is not None:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 
@@ -1479,7 +1531,7 @@ async def send_poll(
         is_anonymous = None, type = None, allows_multiple_answers = None, correct_option_id = None,
         explanation = None, explanation_parse_mode=None, open_period = None, close_date = None, is_closed = None,
         disable_notification=False, reply_to_message_id=None, allow_sending_without_reply=None,
-        reply_markup=None, timeout=None, explanation_entities=None):
+        reply_markup=None, timeout=None, explanation_entities=None, protect_content=None):
     method_url = r'sendPoll'
     payload = {
         'chat_id': str(chat_id),
@@ -1521,6 +1573,8 @@ async def send_poll(
     if explanation_entities:
         payload['explanation_entities'] = json.dumps(
             types.MessageEntity.to_list_of_dicts(explanation_entities))
+    if protect_content:
+        payload['protect_content'] = protect_content
     return await _process_request(token, method_url, params=payload)
 
 async def _convert_list_json_serializable(results):
@@ -1643,4 +1697,10 @@ class ApiTelegramException(ApiException):
             function_name,
             result)
         self.result_json = result_json
-        self.error_code = result_json['error_code']
\ No newline at end of file
+        self.error_code = result_json['error_code']
+
+class RequestTimeout(Exception):
+    """
+    This class represents a request timeout.
+    """
+    pass
\ No newline at end of file
