@@ -12,6 +12,11 @@ import logging
 
 from telebot import types
 
+try:
+    import ujson as json
+except ImportError:
+    import json
+
 try:
     # noinspection PyPackageRequirements
     from PIL import Image
@@ -41,7 +46,7 @@ content_type_service = [
 update_types = [
     "update_id", "message", "edited_message", "channel_post", "edited_channel_post", "inline_query", 
     "chosen_inline_result", "callback_query", "shipping_query", "pre_checkout_query", "poll", "poll_answer", 
-    "my_chat_member", "chat_member"
+    "my_chat_member", "chat_member", "chat_join_request"
 ]
 
 class WorkerThread(threading.Thread):
@@ -165,6 +170,16 @@ class AsyncTask:
             return self.result
 
 
+class CustomRequestResponse():
+    def __init__(self, json_text, status_code = 200, reason = ""):
+        self.status_code = status_code
+        self.text = json_text
+        self.reason = reason
+
+    def json(self):
+        return json.loads(self.text)
+
+
 def async_dec():
     def decorator(fn):
         def wrapper(*args, **kwargs):
@@ -455,3 +470,25 @@ def webhook_google_functions(bot, request):
             return 'Bot FAIL', 400
     else:
         return 'Bot ON'
+
+def antiflood(function, *args, **kwargs):
+    """
+    Use this function inside loops in order to avoid getting TooManyRequests error.
+    Example:
+
+    from telebot.util import antiflood
+    for chat_id in chat_id_list:
+        msg = antiflood(bot.send_message, chat_id, text)
+        
+    You want get the
+    """
+    from telebot.apihelper import ApiTelegramException
+    from time import sleep
+    try:
+        msg = function(*args, **kwargs)
+    except ApiTelegramException as ex:
+        if ex.error_code == 429:
+            sleep(ex.result_json['parameters']['retry_after'])
+            msg = function(*args, **kwargs)
+    finally:
+        return msg
\ No newline at end of file
